C File <cddilog.f> returns the complex value of the function Li2(z)
C with z complex argument.
C It is based on power series of z and inverse z for different regions
C This file is part of aITALC
C Copyright (C) 2003,2004 Alejandro Lorca <alejandro.lorca@desy.de>
C-----------------------------------------------------------------------
      double complex function CDDILOG(z)
C-----------------------------------------------------------------------
      implicit none
      double complex z
      integer nmax,n
      parameter (nmax=200)
      double complex csum(0:nmax), zeta(0:nmax),term
      double precision condition,pi2o6
c      double complex CDLOG
c      print *,'CDABS(z)=',CDABS(z)
C-----------------------------------------------------------------------
      zeta(0)=DCMPLX(1d0,0d0)
      pi2o6=DACOS(-1d0)**2/6
C-----------------------------------------------------------------------
      if (DIMAG(z).EQ.0d0) then
        print *,'CDDILOG: imaginary part EQ 0, something wrong'
        stop
      endif


      if (CDABS(z).EQ.1d0) then
        CDDILOG=DACOS(-1d0)**2/6
C-----------------------------------------------------------------------
      elseif (CDABS(z).LT.0.7d0) then
        csum(0)=DCMPLX(0d0,0d0)
        do n=1,nmax
          zeta(n)=zeta(n-1)*z
          term=zeta(n)/(n**2)
          csum(n)=csum(n-1)+term
          condition=CDABS(term)/CDABS(csum(n))
C        print *,'n=',n,'CDDILOG=',csum(n)
          if (condition.LT.1D-16) goto 10
          if (n.EQ.nmax) print *,'CDDILOG: nmax reached, converg |z|<=1'
        enddo
c 10     CDDILOG= csum(n)
C-----------------------------------------------------------------------
      elseif ((CDABS(z).GE.0.7d0).AND.(CDABS(z).LE.1.4d0)) then
        if (DBLE(z).GE.0d0) then
C Next I believe is a bug in the compiler.
c          csum(0)=(-CDLOG(1-z)*CDLOG(z)+DACOS(-1d0)**2/6)
c         print *,'csum(0)=',csum(0)
          csum(0)=-LOG(1-z)*LOG(z)+pi2o6
c         print *,'csum(0)=',csum(0)
          do n=1,nmax
            zeta(n)=zeta(n-1)*(1-z)
            term=-zeta(n)/(n**2)
            csum(n)=csum(n-1)+term
            condition=CDABS(term)/CDABS(csum(n))
c       print *,'n=',n,'CDDILOG=',csum(n)
            if (condition.LT.1D-16) goto 10
            if (n.EQ.nmax) print *
     &           ,'CDDILOG: nmax reached, converg |z|=1'
          enddo
        else
C The next strategy for values like (-1,epsilon) is to shift the
C calculation to (2,epsilon)
          z=1-z
          csum(0)=-LOG(-z)**2/2-pi2o6
          do n=1,nmax
            zeta(n)=zeta(n-1)/z
            term=-zeta(n)/(n**2)
            csum(n)=csum(n-1)+term
            condition=CDABS(term)/CDABS(csum(n))
C        print *,'n=',n,'CDDILOG=',csum(n)
            if (condition.LT.1D-16) then
              csum(n)=-csum(n)-LOG(z)*LOG(1-z)+pi2o6
              goto 10
            endif
            if (n.EQ.nmax) print *
     &           ,'CDDILOG: nmax reached, converg |z|>=1'
        enddo
      endif
      else
        csum(0)=-LOG(-z)**2/2-pi2o6
        do n=1,nmax
          zeta(n)=zeta(n-1)/z
          term=-zeta(n)/(n**2)
          csum(n)=csum(n-1)+term
          condition=CDABS(term)/CDABS(csum(n))
C        print *,'n=',n,'CDDILOG=',csum(n)
          if (condition.LT.1D-16) goto 10
          if (n.EQ.nmax) print *,'CDDILOG: nmax reached, converg |z|>=1'
        enddo
      endif
      
 10   CDDILOG = csum(n)

      return
      end
C***********************************************************************

      double complex function CDDILOGMU(z,musq,i)
      implicit none
      double complex z
      double precision musq
      integer i
      double complex DILOGIE,CDDILOG
C-----------------------------------------------------------------------
      if (DIMAG(z).EQ.0d0) then
C Evaluate with dilogie
        CDDILOGMU= DILOGIE(DBLE(z)/musq,i)
      else
C Evaluate with cddilog
        CDDILOGMU= CDDILOG(z/DCMPLX(musq))
      endif
C-----------------------------------------------------------------------
      return
      end

C $Id: cddilog.f,v 1.6 2004/10/15 19:19:06 alorca Exp $
